print("import: 'mdit_py_plugins'")
import mdit_py_plugins

print("import: 'mdit_py_plugins.amsmath'")
import mdit_py_plugins.amsmath

print("import: 'mdit_py_plugins.anchors'")
import mdit_py_plugins.anchors

print("import: 'mdit_py_plugins.container'")
import mdit_py_plugins.container

print("import: 'mdit_py_plugins.deflist'")
import mdit_py_plugins.deflist

print("import: 'mdit_py_plugins.dollarmath'")
import mdit_py_plugins.dollarmath

print("import: 'mdit_py_plugins.footnote'")
import mdit_py_plugins.footnote

print("import: 'mdit_py_plugins.front_matter'")
import mdit_py_plugins.front_matter

print("import: 'mdit_py_plugins.myst_blocks'")
import mdit_py_plugins.myst_blocks

print("import: 'mdit_py_plugins.myst_role'")
import mdit_py_plugins.myst_role

print("import: 'mdit_py_plugins.tasklists'")
import mdit_py_plugins.tasklists

print("import: 'mdit_py_plugins.texmath'")
import mdit_py_plugins.texmath

